@@ -36,12 +36,19 @@
 
 /*lint -save -e525 -e539 -e835 */
 
-/*--------------------------------------------------------------------
- *  * Simple parameters
- *   */
+#ifndef PARAM_ALL
+#  define PARAM_PRE
+#  define PARAM_POST
+#endif
 
-#define PARAM_SIMPLE(nm, typ, ...) \
-	PARAM(typ, nm, nm, tweak_##typ, &mgt_param.nm, __VA_ARGS__)
+/*--------------------------------------------------------------------
+ * Simple parameters
+ */
+
+#define PARAM_SIMPLE(nm, typ, ...)					\
+	PARAM_PRE							\
+	PARAM(typ, nm, nm, tweak_##typ, &mgt_param.nm, __VA_ARGS__)	\
+	PARAM_POST
 
 #if defined(PLATFORM_FLAGS)
 #  error "Temporary macro PLATFORM_FLAGS already defined"
@@ -325,7 +332,7 @@ PARAM_SIMPLE(
 	/* def */	"60.000",
 	/* units */	"seconds",
 	/* descr */
-	"Timeout for the childs replies to CLI requests from the "
+	"Timeout for the child's replies to CLI requests from the "
 	"mgt_param."
 )
 
@@ -385,13 +392,16 @@ PARAM_SIMPLE(
 	/* type */	timeout,
 	/* min */	"0.000",
 	/* max */	NULL,
-	/* def */	"10.000",
+	/* def */	"10s",
 	/* units */	"seconds",
 	/* descr */
 	"Default grace period.  We will deliver an object this long after "
 	"it has expired, provided another thread is attempting to get a "
 	"new copy.",
-	/* flags */	OBJ_STICKY
+	/* flags */	OBJ_STICKY,
+	/* dyn_min_reason */	NULL,
+	/* dyn_max_reason */	NULL,
+	/* dyn_def_reason */	"10s"
 )
 
 PARAM_SIMPLE(
@@ -399,14 +409,17 @@ PARAM_SIMPLE(
 	/* type */	timeout,
 	/* min */	"0.000",
 	/* max */	NULL,
-	/* def */	"0.000",
+	/* def */	"0s",
 	/* units */	"seconds",
 	/* descr */
 	"Default keep period.  We will keep a useless object around this "
 	"long, making it available for conditional backend fetches.  That "
 	"means that the object will be removed from the cache at the end "
 	"of ttl+grace+keep.",
-	/* flags */	OBJ_STICKY
+	/* flags */	OBJ_STICKY,
+	/* dyn_min_reason */	NULL,
+	/* dyn_max_reason */	NULL,
+	/* dyn_def_reason */	"0s"
 )
 
 PARAM_SIMPLE(
@@ -414,12 +427,15 @@ PARAM_SIMPLE(
 	/* type */	timeout,
 	/* min */	"0.000",
 	/* max */	NULL,
-	/* def */	"120.000",
+	/* def */	"2m",
 	/* units */	"seconds",
 	/* descr */
 	"The TTL assigned to objects if neither the backend nor the VCL "
 	"code assigns one.",
-	/* flags */	OBJ_STICKY
+	/* flags */	OBJ_STICKY,
+	/* dyn_min_reason */	NULL,
+	/* dyn_max_reason */	NULL,
+	/* dyn_def_reason */	"2m"
 )
 
 PARAM_SIMPLE(
@@ -610,11 +626,6 @@ PARAM_SIMPLE(
 	"of that the response is allowed to take up."
 )
 
-#if defined(SO_SNDTIMEO_WORKS)
-#  define PLATFORM_FLAGS DELAYED_EFFECT
-#else
-#  define PLATFORM_FLAGS NOT_IMPLEMENTED
-#endif
 PARAM_SIMPLE(
 	/* name */	idle_send_timeout,
 	/* type */	timeout,
@@ -628,9 +639,8 @@ PARAM_SIMPLE(
 	"When this timeout is hit, the session is closed.\n\n"
 	"See the man page for `setsockopt(2)` or `socket(7)` under"
 	" ``SO_SNDTIMEO`` for more information.",
-	/* flags */	PLATFORM_DEPENDENT | PLATFORM_FLAGS
+	/* flags */	DELAYED_EFFECT
 )
-#undef PLATFORM_FLAGS
 
 PARAM_SIMPLE(
 	/* name */	listen_depth,
@@ -781,11 +791,6 @@ PARAM_SIMPLE(
 	/* flags */	EXPERIMENTAL
 )
 
-#if defined(SO_SNDTIMEO_WORKS)
-#  define PLATFORM_FLAGS DELAYED_EFFECT
-#else
-#  define PLATFORM_FLAGS NOT_IMPLEMENTED
-#endif
 PARAM_SIMPLE(
 	/* name */	send_timeout,
 	/* type */	timeout,
@@ -800,9 +805,8 @@ PARAM_SIMPLE(
 	" timeout is extended unless the total time already taken for sending"
 	" the response in its entirety exceeds this many seconds.\n\n"
 	"When this timeout is hit, the session is closed",
-	/* flags */	PLATFORM_DEPENDENT | PLATFORM_FLAGS
+	/* flags */	DELAYED_EFFECT
 )
-#undef PLATFORM_FLAGS
 
 PARAM_SIMPLE(
 	/* name */	shortlived,
@@ -914,11 +918,6 @@ PARAM_SIMPLE(
 )
 #undef PLATFORM_FLAGS
 
-#if defined(SO_RCVTIMEO_WORKS)
-#  define PLATFORM_FLAGS 0
-#else
-#  define PLATFORM_FLAGS NOT_IMPLEMENTED
-#endif
 PARAM_SIMPLE(
 	/* name */	timeout_idle,
 	/* type */	timeout,
@@ -932,10 +931,8 @@ PARAM_SIMPLE(
 	" request headers.\n\n"
 	"This parameter is particularly relevant for HTTP1 keepalive "
 	" connections which are closed unless the next request is received"
-	" before this timeout is reached.",
-	/* flags */	PLATFORM_DEPENDENT | PLATFORM_FLAGS
+	" before this timeout is reached."
 )
-#undef PLATFORM_FLAGS
 
 PARAM_SIMPLE(
 	/* name */	timeout_linger,
@@ -955,6 +952,31 @@ PARAM_SIMPLE(
 	/* flags */	EXPERIMENTAL
 )
 
+PARAM_SIMPLE(
+	/* name */	transit_buffer,
+	/* type */	bytes,
+	/* min */	"0k",
+	/* max */	NULL,
+	/* def */	"0k",
+	/* units */	"bytes",
+	/* descr */
+
+	"The number of bytes which Varnish buffers for uncacheable "
+	"backend streaming fetches - in other words, how many bytes "
+	"Varnish reads from the backend ahead of what has been sent to "
+	"the client.\n"
+	"A zero value means no limit, the object is fetched as fast as "
+	"possible.\n\n"
+	"When dealing with slow clients, setting this parameter to "
+	"non-zero can prevent large uncacheable objects from being "
+	"stored in full when the intent is to simply stream them to the "
+	"client. As a result, a slow client transaction holds onto a "
+	"backend connection until the end of the delivery.\n\n"
+	"This parameter is the default to the VCL variable "
+	"``beresp.transit_buffer``, which can be used to control the "
+	"transit buffer per backend request."
+)
+
 PARAM_SIMPLE(
 	/* name */	vary_notice,
 	/* type */	uint,
@@ -1040,7 +1062,7 @@ PARAM_SIMPLE(
 	/* name */	vsl_reclen,
 	/* type */	vsl_reclen,
 	/* min */	"16b",
-	/* max */	NULL,
+	/* max */	"65535b",	// VSL_LENMASK
 	/* def */	"255b",
 	/* units */	"bytes",
 	/* descr */
@@ -1065,18 +1087,6 @@ PARAM_SIMPLE(
 	/* flags */	MUST_RESTART
 )
 
-PARAM_SIMPLE(
-	/* name */	vsm_space,
-	/* type */	bytes,
-	/* min */	"1M",
-	/* max */	"1G",
-	/* def */	"1M",
-	/* units */	"bytes",
-	/* descr */
-	"DEPRECATED: This parameter is ignored.\n"
-	"There is no global limit on amount of shared memory now."
-)
-
 PARAM_SIMPLE(
 	/* name */	workspace_backend,
 	/* type */	bytes_u,
@@ -1198,7 +1208,8 @@ PARAM_SIMPLE(
 /* We have a strict min at the protocol default here. This is because we
  * don't have the 'use settings only after peer ack' in place yet. If the
  * value is lower than the protocol default, the very first stream could
- * get a flow control error. */
+ * get a flow control error.
+ */
 PARAM_SIMPLE(
 	/* name */	h2_initial_window_size,
 	/* type */	bytes_u,
@@ -1237,13 +1248,15 @@ PARAM_SIMPLE(
  */
 
 #define PARAM_MEMPOOL(nm, def, descr)					\
+	PARAM_PRE							\
 	PARAM(poolparam, nm, nm, tweak_poolparam, &mgt_param.nm,	\
 	    NULL, NULL, def, NULL,					\
 	    descr							\
 	    "The three numbers are:\n"					\
 	    "\tmin_pool\tminimum size of free pool.\n"			\
 	    "\tmax_pool\tmaximum size of free pool.\n"			\
-	    "\tmax_age\tmax age of free element.")
+	    "\tmax_age\tmax age of free element.")			\
+	PARAM_POST
 
 PARAM_MEMPOOL(
 		/* name */	pool_req,
@@ -1271,8 +1284,10 @@ PARAM_MEMPOOL(
  */
 
 #define PARAM_THREAD(nm, fld, typ, ...)			\
+	PARAM_PRE					\
 	PARAM(typ, wthread_ ## fld, nm, tweak_ ## typ,	\
-	    &mgt_param.wthread_ ## fld, __VA_ARGS__)
+	    &mgt_param.wthread_ ## fld, __VA_ARGS__)	\
+	PARAM_POST
 
 PARAM_THREAD(
 	/* name */	thread_pools,
@@ -1322,7 +1337,7 @@ PARAM_THREAD(
 	/* name */	thread_pool_min,
 	/* field */	min,
 	/* type */	thread_pool_min,
-	/* min */	"5" /* TASK_QUEUE__END */,
+	/* min */	"5" /* TASK_QUEUE_RESERVE */,
 	/* max */	NULL,
 	/* def */	"100",
 	/* units */	"threads",
@@ -1333,9 +1348,9 @@ PARAM_THREAD(
 	"situations or when threads have expired.\n"
 	"\n"
 	"Technical minimum is 5 threads, but this parameter is "
-	/*                    ^ TASK_QUEUE__END */
+	/*                    ^ TASK_QUEUE_RESERVE */
 	"strongly recommended to be at least 10",
-	/*               2 * TASK_QUEUE__END ^^ */
+	/*            2 * TASK_QUEUE_RESERVE ^^ */
 	/* flags */	DELAYED_EFFECT,
 	/* dyn_min_reason */	NULL,
 	/* dyn_max_reason */	"thread_pool_max"
@@ -1360,7 +1375,7 @@ PARAM_THREAD(
 	"priority tasks from running even under high load.\n"
 	"\n"
 	"The effective value is at least 5 (the number of internal "
-	/*                               ^ TASK_QUEUE__END */
+	/*                               ^ TASK_QUEUE_RESERVE */
 	"priority classes), irrespective of this parameter.",
 	/* flags */	DELAYED_EFFECT,
 	/* dyn_min_reason */	NULL,
@@ -1551,8 +1566,10 @@ PARAM_THREAD(
  * String parameters
  */
 
-#  define PARAM_STRING(nm, tw, pv, def, ...) \
-	PARAM(, , nm, tw, pv, NULL, NULL, def, NULL, __VA_ARGS__)
+#  define PARAM_STRING(nm, tw, pv, def, ...)				\
+	PARAM_PRE							\
+	PARAM(, , nm, tw, pv, NULL, NULL, def, NULL, __VA_ARGS__)	\
+	PARAM_POST
 
 PARAM_STRING(
 	/* name */	cc_command,
@@ -1560,11 +1577,42 @@ PARAM_STRING(
 	/* priv */	&mgt_cc_cmd,
 	/* def */	VCC_CC,
 	/* descr */
-	"Command used for compiling the C source code to a "
-	"dlopen(3) loadable object.  Any occurrence of %s in "
-	"the string will be replaced with the source file name, "
-	"and %o will be replaced with the output file name.",
-	/* flags */	MUST_RELOAD
+	"The command used for compiling the C source code to a "
+	"dlopen(3) loadable object. The following expansions can "
+	"be used:\n\n"
+	"- %s: the source file name\n"
+	"- %o: the output file name\n"
+	"- %w: the cc_warnings parameter\n"
+	"- %d: the raw default cc_command\n"
+	"- %D: the expanded default cc_command\n"
+	"- %n: the working directory (-n option)\n"
+	"- %%: a percent sign\n"
+	"\n"
+	"Unknown percent expansion sequences are ignored, and to "
+	"avoid future incompatibilities percent characters should "
+	"be escaped with a double percent sequence.\n\n"
+	"The %d and %D expansions allow passing the parameter's "
+	"default value to a wrapper script to perform additional "
+	"processing.",
+	/* flags */	MUST_RELOAD | BUILD_OPTIONS,
+	/* dyn_min_reason */	NULL,
+	/* dyn_max_reason */	NULL,
+	/* dyn_def_reason */	"exec $CC $CFLAGS %w -shared -o %o %s"
+)
+
+PARAM_STRING(
+	/* name */	cc_warnings,
+	/* tweak */	tweak_string,
+	/* priv */	&mgt_cc_warn,
+	/* def */	VCC_WARN,
+	/* descr */
+	"Warnings used when compiling the C source code with "
+	"the cc_command parameter. By default, VCL is compiled "
+	"with the same set of warnings as Varnish itself.",
+	/* flags */	MUST_RELOAD | BUILD_OPTIONS,
+	/* dyn_min_reason */	NULL,
+	/* dyn_max_reason */	NULL,
+	/* dyn_def_reason */	"-Wall -Werror"
 )
 
 PARAM_STRING(
@@ -1590,7 +1638,11 @@ PARAM_STRING(
 	"VCL files in both the system configuration and shared "
 	"data directories to allow packages to drop their VCL "
 	"files in a standard location where relative includes "
-	"would work."
+	"would work.",
+	/* flags */	BUILD_OPTIONS,
+	/* dyn_min_reason */	NULL,
+	/* dyn_max_reason */	NULL,
+	/* dyn_def_reason */	"${sysconfdir}/varnish:${datadir}/varnish/vcl"
 )
 
 PARAM_STRING(
@@ -1600,36 +1652,11 @@ PARAM_STRING(
 	/* def */	VARNISH_VMOD_DIR,
 	/* descr */
 	"Directory (or colon separated list of directories) "
-	"where VMODs are to be found."
-)
-
-/*--------------------------------------------------------------------
- * VCC parameters
- */
-
-#  define PARAM_VCC(nm, def, descr) \
-	PARAM(, , nm, tweak_boolean, &mgt_ ## nm, NULL, NULL, def, "bool", descr)
-
-PARAM_VCC(
-	/* name */	vcc_err_unref,
-	/* def */	"on",
-	/* descr */
-	"Unreferenced VCL objects result in error."
-)
-
-PARAM_VCC(
-	/* name */	vcc_allow_inline_c,
-	/* def */	"off",
-	/* descr */
-	"Allow inline C code in VCL."
-)
-
-PARAM_VCC(
-	/* name */	vcc_unsafe_path,
-	/* def */	"on",
-	/* descr */
-	"Allow '/' in vmod & include paths.\n"
-	"Allow 'import ... from ...'."
+	"where VMODs are to be found.",
+	/* flags */	BUILD_OPTIONS,
+	/* dyn_min_reason */	NULL,
+	/* dyn_max_reason */	NULL,
+	/* dyn_def_reason */	"${libdir}/varnish/vmods"
 )
 
 /*--------------------------------------------------------------------
@@ -1637,8 +1664,10 @@ PARAM_VCC(
  */
 
 #  define PARAM_PCRE2(nm, pv, min, def, descr)			\
+	PARAM_PRE						\
 	PARAM(, , nm, tweak_uint, &mgt_param.vre_limits.pv,	\
-	    min, NULL, def, NULL, descr)
+	    min, NULL, def, NULL, descr)			\
+	PARAM_POST
 
 PARAM_PCRE2(
 	/* name */	pcre2_match_limit,
@@ -1671,83 +1700,138 @@ PARAM_PCRE2(
 	" messages."
 )
 
-#  undef PARAM_ALL
+/*--------------------------------------------------------------------
+ * Parameter deprecated aliases
+ *
+ * When a parameter is renamed, but a deprecated alias is kept for
+ * compatibility, its documentation is minimal: only a description in
+ * manual pages, a description and current value in the CLI.
+ *
+ * The deprecated_dummy alias is here for test coverage.
+ */
+
+#define PARAM_ALIAS(al, nm)					\
+	PARAM_PRE						\
+	PARAM(, , al, tweak_alias, NULL, NULL, NULL, #nm, NULL, \
+	    "Deprecated alias for the " #nm " parameter.")	\
+	PARAM_POST
+
+PARAM_ALIAS(deprecated_dummy,	debug)
+PARAM_ALIAS(vcc_err_unref,	vcc_feature)
+PARAM_ALIAS(vcc_allow_inline_c,	vcc_feature)
+PARAM_ALIAS(vcc_unsafe_path,	vcc_feature)
+
+#  undef PARAM_ALIAS
 #  undef PARAM_PCRE2
 #  undef PARAM_STRING
-#  undef PARAM_VCC
 #endif /* defined(PARAM_ALL) */
 
+/*--------------------------------------------------------------------
+ * Bits parameters
+ */
+
+#define PARAM_BITS(nm, fld, def, descr)					\
+	PARAM(nm, fld, nm, tweak_ ## nm, mgt_param.fld, NULL, NULL,	\
+	    def, NULL, descr)
+
+PARAM_PRE
+PARAM_BITS(
+	/* name */	debug,
+	/* fld */	debug_bits,
+	/* def */	"none",
+	/* descr */
+	"Enable/Disable various kinds of debugging.\n"
+	"\tnone\tDisable all debugging\n\n"
+	"Use +/- prefix to set/reset individual bits:")
+#ifdef PARAM_ALL
+#  define DEBUG_BIT(U, l, d) "\n\t" #l "\t" d
+#  include "tbl/debug_bits.h"
+#endif
+PARAM_POST
+
+PARAM_PRE
+PARAM_BITS(
+	/* name */	experimental,
+	/* fld */	experimental_bits,
+	/* def */	"none",
+	/* descr */
+	"Enable/Disable experimental features.\n"
+	"\tnone\tDisable all experimental features\n\n"
+	"Use +/- prefix to set/reset individual bits:")
+#ifdef PARAM_ALL
+#  define EXPERIMENTAL_BIT(U, l, d) "\n\t" #l "\t" d
+#  include "tbl/experimental_bits.h"
+#endif
+PARAM_POST
+
+PARAM_PRE
+PARAM_BITS(
+	/* name */	feature,
+	/* fld */	feature_bits,
+	/* def */	"+validate_headers",
+	/* descr */
+	"Enable/Disable various minor features.\n"
+	"\tdefault\tSet default value\n"
+	"\tnone\tDisable all features.\n\n"
+	"Use +/- prefix to enable/disable individual feature:")
+#ifdef PARAM_ALL
+#  define FEATURE_BIT(U, l, d) "\n\t" #l "\t" d
+#  include "tbl/feature_bits.h"
+#endif
+PARAM_POST
+
+PARAM_PRE
+PARAM_BITS(
+	/* name */	vcc_feature,
+	/* fld */	vcc_feature_bits,
+	/* def */
+	"+err_unref,"
+	"+unsafe_path",
+	/* descr */
+	"Enable/Disable various VCC behaviors.\n"
+	"\tdefault\tSet default value\n"
+	"\tnone\tDisable all behaviors.\n\n"
+	"Use +/- prefix to enable/disable individual behavior:")
+#ifdef PARAM_ALL
+#  define VCC_FEATURE_BIT(U, l, d) "\n\t" #l "\t" d
+#  include "tbl/vcc_feature_bits.h"
+#endif
+PARAM_POST
+
+PARAM_PRE
+PARAM_BITS(
+	/* name */	vsl_mask,
+	/* fld */	vsl_mask,
+	/* def */
+	"-Debug,"
+	"-ExpKill,"
+	"-H2RxBody,"
+	"-H2RxHdr,"
+	"-H2TxBody,"
+	"-H2TxHdr,"
+	"-Hash,"
+	"-ObjHeader,"
+	"-ObjProtocol,"
+	"-ObjReason,"
+	"-ObjStatus,"
+	"-VCL_trace,"
+	"-VdpAcct,"
+	"-VfpAcct,"
+	"-WorkThread",
+	/* descr */
+	"Mask individual VSL messages from being logged.\n"
+	"\tdefault\tSet default value\n"
+	"\nUse +/- prefix in front of VSL tag name to unmask/mask "
+	"individual VSL messages.")
+PARAM_POST
+
+#undef PARAM_ALL
+#undef PARAM_BITS
 #undef PARAM_MEMPOOL
+#undef PARAM_POST
+#undef PARAM_PRE
 #undef PARAM_SIMPLE
 #undef PARAM_THREAD
 #undef PARAM
 
-#if 0 /* NOT ACTUALLY DEFINED HERE */
-/* actual location mgt_param_bits.c*/
-/* see tbl/debug_bits.h */
-PARAM(
-	/* name */	debug,
-	/* type */	debug,
-	/* min */	NULL,
-	/* max */	NULL,
-	/* def */	NULL,
-	/* units */	NULL,
-	/* descr */
-	"Enable/Disable various kinds of debugging.\n"
-	"	none	Disable all debugging\n"
-	"\n"
-	"Use +/- prefix to set/reset individual bits:\n"
-	"	req_state	VSL Request state engine\n"
-	"	workspace	VSL Workspace operations\n"
-	"	waiter	VSL Waiter internals\n"
-	"	waitinglist	VSL Waitinglist events\n"
-	"	syncvsl	Make VSL synchronous\n"
-	"	hashedge	Edge cases in Hash\n"
-	"	vclrel	Rapid VCL release\n"
-	"	lurker	VSL Ban lurker\n"
-	"	esi_chop	Chop ESI fetch to bits\n"
-	"	flush_head	Flush after http1 head\n"
-	"	vtc_mode	Varnishtest Mode"
-)
-
-/* actual location mgt_param_bits.c*/
-/* See tbl/feature_bits.h */
-PARAM(
-	/* name */	feature,
-	/* type */	feature,
-	/* min */	NULL,
-	/* max */	NULL,
-	/* def */	NULL,
-	/* units */	NULL,
-	/* descr */
-	"Enable/Disable various minor features.\n"
-	"	none	Disable all features.\n"
-	"\n"
-	"Use +/- prefix to enable/disable individual feature:\n"
-	"	short_panic	Short panic message.\n"
-	"	wait_silo	Wait for persistent silo.\n"
-	"	no_coredump	No coredumps.\n"
-	"	esi_ignore_https	Treat HTTPS as HTTP in ESI:includes\n"
-	"	esi_disable_xml_check	Don't check of body looks like XML\n"
-	"	esi_ignore_other_elements	Ignore non-esi XML-elements\n"
-	"	esi_remove_bom	Remove UTF-8 BOM"
-)
-
-/* actual location mgt_param_bits.c*/
-PARAM(
-	/* name */	vsl_mask,
-	/* type */	vsl_mask,
-	/* min */	NULL,
-	/* max */	NULL,
-	/* def */	"default",
-	/* units */	NULL,
-	/* descr */
-	"Mask individual VSL messages from being logged.\n"
-	"	default	Set default value\n"
-	"\n"
-	"Use +/- prefix in front of VSL tag name to unmask/mask "
-	"individual VSL messages."
-)
-#endif /* NOT ACTUALLY DEFINED HERE */
-
 /*lint -restore */
